/************************************************************************
 *
 *  (C) Copyright 2002-2008 Novell, Inc.
 *  All Rights Reserved.
 *
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Novell, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Novell, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 ****************************************************************************/
#ifndef __SPM_H
#define __SPM_H

#include <nmasapi.h>
#include <nmasflgs.h>

#define SPM_MAX_PASSWORD_CHARS 512

/* Password Storage Types */
#define SPM_TYPE_UNICODE            0x80000000

/* Flags for the nspmConfigurationOptions attribute*/
#define SPM_DELETE_HSP              0x00000001     
#define SPM_DISABLE_HSP_SYNC        0x00000002
#define SPM_DISABLE_SPWD_SYNC       0x00000004
#define SPM_ALLOW_SPWD_SET          0x00000008
#define SPM_ALLOW_SELF_RETRIEVAL    0x00000010
#define SPM_ALLOW_ADMIN_RETRIEVAL   0x00000020
#define SPM_ALLOW_EXTERNAL_SYNC     0x00000040
#define SPM_ENABLED                 0x00000100
#define SPM_POLICY_ENABLED          0x00000200
#define SPM_CONFIG_INVALID          0x80000000L

// Flags that may be passed to the SPM set password function from a SPM agent
#define SPM_ADMIN_REQUEST   0x1       // Indicates that the password change request is not from the user
#define SPM_SET_LEGACY_PWD  0x2       // Indicates that the password change request should also set legacy password
#define SPM_DEL_LEGACY_PWD  0x4       // Indicates that the password change request should also remove legacy password
#define SPM_IGNORE_POLICY   0x8       // Indicates that the password policy will not be checked on the input password
#define SPM_EXPIRE_NOW      0x10      // Indicates that the new password will be marked expired
#define SPM_RESERVED        0x20      // Reserved for future use
#define SPM_DIST_SYNC       0x40      // Indicates that SPM password and the distribution will be set to the same value
#define SPM_SPWD_SYNC       0x80      // Indicates that SPM password and the simple password will be set to the same value
#define SPM_MIGRATION       0x100     // Indicates that the password is being migrated from another password

// Flags that may be passed to the SPM get password and get password history functions
#define SPM_SYNC_REQUEST        0x1 // password expiration and grace login are not enforced
#define SPM_INFO_REQUEST        0x2 // only password information (length, creation time, expiration and grace logins) are returned
#define SPM_DONT_MIGRATE_SPWD   0x4 // simple password is not to be copied to password
#define SPM_IGNORE_NDS          0x8 // don't check if NDS password is newer than password
#define SPM_IGNORE_EXPIRATION  0x10 // don't read or check password expiratiopn and grace login

// Flags that may be returned from the SPM get password function
#define SPM_EXPIRED             0x1
#define SPM_GRACE_LOGIN_LIMIT   0x2
#define SPM_SPWD_MIGRATED       0x4     // simple password was copied to password -- returned only by MAF_GetPassword
#define SPM_CASE_INSENSITIVE    0x8     // case insensitive password is enabled  -- returned only by MAF_GetPassword
#define SPM_NOT_COMPLIANT       0x10    // pwd does not comply to pwd policy  -- returned only by MAF_GetPassword

// Password lifetime value that indicates that the does not have an expiration time
#define SPM_NO_PASSWORD_EXPIRATION  0xFFFFFFFF

// attribute value in grace login limit that indicates that there is no grace login limit
#define SPM_UNLIMITED_GRACE_LOGIN   0xff

/* ********************************************************************************************** */
//
// SPM Password Agent API
//
/* ********************************************************************************************** */
// version for the SPM function structure
#define SPM_FUNCTION_VERSION   0x2
#define SPM_FUNCTION_VERSION1  0x1

typedef struct SPM_Functions
{
    int structureVersion;
    int (*registerAgent)(unicode *agentName);
    int (*deregisterAgent)(unicode *agentName);
    int (*getPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nuint32 *remainingLifetime, nuint32 *remainingGraceLogins);
    int (*getPasswordHistory)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, nuint32 *historyEnabled, nuint32 *passwordCount, size_t *passwordBufferSize, unicode *passwordBuffer);
    int (*setPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, size_t passwordLen, unicode *password, nuint32 setFlags);
    int (*chgPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, size_t oldPasswordLen, unicode *oldPassword, size_t newPasswordLen, unicode *newPassword);
    int (*policyCheckPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, size_t passwordLen, unicode *password);
    int (*initDecryptPasswordValue)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, void **handle);
    int (*decryptPasswordValue)(void *handle, size_t valueLen, void *value, size_t *passwordLen, unicode *password);
    int (*closeDecryptPasswordValue)(void *handle);
} SPM_Functions;

NMASAPI int SPM_Agent(
    unicode *agentName,
    SPM_Functions *functions);

NMASAPI int SPM_AgentVer(
    unicode *agentName,
    int version,
    SPM_Functions *functions);


/* ********************************************************************************************** */
//
// SPM Policy Agent API
//
/* ********************************************************************************************** */

// In addition to all of the flags that may be passed into the SPM set password function,
// the following flags that may be passed to SPM policy agent policy callback functions and
// SPM change agent call back functions from SPM
#define SPM_CURRENT_PASSWORD    0x8000000   // Indicates that the password is the user's active password.
#define SPM_CHECK_PASSWORD_ONLY 0x4000000   // Indicates that request is a password policy check (not change)
#define SPM_DIST_PASSWORD_ONLY  0x2000000   // Indicates that request is for the distribution password only
#define SPM_DELETE_PASSWORD     0x1000000   // Indicates that request is for deleting the password

typedef int SPM_PolicyCallBackFunction (
    unicode *agentName,             // name of the agent that requested the change, may be NULL
    unicode *subjName,              // DN of the identity that requested the change, may be NULL
    unicode *treeName,              // Name of the tree in which the user resides
    unicode *objectDN,              // DN of the user
    unicode *passwordPolicyDN,      // DN of password policy effective for the user
    nuint32 flags,                  // Password set flags (see flag definitions above)
    size_t candidatePasswordLen,    // length of password in characters including the NULL termination
    unicode *candidatePassword);    // password that will be set as the new password

typedef int SPM_CheckModifyRightsFunction (
    unicode *agentName,         // name of the agent that requested the change, may be NULL
    unicode *subjName,          // DN of the identity that requested the change, may be NULL
    unicode *treeName,          // Name of the tree in which the user resides
    unicode *objectDN,          // DN of the user
    unicode *passwordPolicyDN); // DN of password policy effective for the user

typedef int SPM_CheckReadRightsFunction (
    unicode *agentName,         // name of the agent that requested the change, may be NULL
    unicode *subjName,          // DN of the identity that requested the change, may be NULL
    unicode *treeName,          // Name of the tree in which the user resides
    unicode *objectDN,          // DN of the user
    unicode *passwordPolicyDN); // DN of password policy effective for the user

#define SPM_POLICY_FUNCTION_VERSION 0x1

typedef struct SPM_PolicyFunctions
{
    int structureVersion;
    int (*registerAgent)(unicode *agentName, SPM_PolicyCallBackFunction *policyCallback, SPM_CheckModifyRightsFunction *modifyCheckCallBack, SPM_CheckReadRightsFunction *readCheckCallBack);
    int (*deregisterAgent)(unicode *agentName);
    int (*getPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nuint32 *remainingLifetime, nuint32 *remainingGraceLogins);
    int (*getPasswordHistory)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, nuint32 *historyEnabled, nuint32 *passwordCount, size_t *passwordBufferSize, unicode *passwordBuffer);
    int (*getPasswordCreateTime)(unicode *agentName, unicode *treeName, unicode *objectDN, nuint32 *createTime);
} SPM_PolicyFunctions;

NMASAPI int SPM_PolicyAgent(
    unicode *agentName,
    SPM_PolicyFunctions *functions);

NMASAPI int SPM_PolicyAgentVer(
    unicode *agentName,
    int version,
    SPM_PolicyFunctions *functions);


/* ********************************************************************************************** */
//
// SPM Distribution Agent API
//
/* ********************************************************************************************** */

#define SPM_DIST_FUNCTION_VERSION   0x3
#define SPM_DIST_FUNCTION_VERSION2  0x2
#define SPM_DIST_FUNCTION_VERSION1  0x1
typedef struct SPM_DistFunctions
{
    int structureVersion;
    int (*registerAgent)(unicode *agentName);
    int (*deregisterAgent)(unicode *agentName);
    int (*getPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, size_t *passwordLen, unicode *password);
    int (*setPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, size_t passwordLen, unicode *password, nuint32 setFlags);
    int (*policyCheckPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, size_t passwordLen, unicode *password);
    int (*getPreviousPassword)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, nuint32 getFlags, size_t *passwordLen, unicode *password);
    int (*initDecryptPasswordValue)(unicode *agentName, unicode *subjName, unicode *treeName, unicode *objectDN, void **handle);
    int (*decryptPasswordValue)(void *handle, size_t valueLen, void *value, size_t *passwordLen, unicode *password);
    int (*closeDecryptPasswordValue)(void *handle);
} SPM_DistFunctions;

NMASAPI int SPM_DistributionAgent(
    unicode *agentName,
    SPM_DistFunctions *functions);

NMASAPI int SPM_DistributionAgentVer(
    unicode *agentName,
    int version,
    SPM_DistFunctions *functions);


/* ********************************************************************************************** */
//
// SPM Change Agent API
//
/* ********************************************************************************************** */

#define SPMCHANGE_CONTEXT void*
#define INVALID_SPMCHANGE_CONTEXT ((SPMCHANGE_CONTEXT)-1)

#define SPMDS_MAX_DN_CHARS 256

#define SPMDS_VALUES void*

/* Value types for reading and writing attributes. */
#define SPMDS_SYN_DISTNAME     1    
#define SPMDS_SYN_CE_STRING    2    /* Case-exact string */
#define SPMDS_SYN_CI_STRING    3    /* Case-ignore string */
#define SPMDS_SYN_BOOLEAN      4     
#define SPMDS_SYN_INTEGER      5     
#define SPMDS_SYN_OCTET_STRING 6    /* Buffer with binary data. */
#define SPMDS_SYN_TIME         7    /* Four-byte time value (nuint32) */

/* Values for "operation" parameter in insertModValue */
#define SPMDS_MODOP_ADD_VALUE    1    /* Add a value to an attribute. (Creates the attribute if it does not exist.) */
#define SPMDS_MODOP_MODIFY_VALUE 2 /* Change a value for a single-valued attribute, or add a value to a multi-valued attribute. */
#define SPMDS_MODOP_DEL_VALUE    3    /* Remove a value from an attribute */
#define SPMDS_MODOP_DEL_ATTR     4     /* Remove an attribute and all of its values. */

typedef struct SPMDS_ATTRIBUTE
{
    unicode *attr;
    int     syntax;
}SPMDS_ATTRIBUTE;

typedef struct SPMDS_VALUE_DATA
{
    /* The value */
    void         *data;
    size_t       size;
    
    /* The attribute from which this value was read. */
    unicode     *attr;
    int         syntax;

}SPMDS_VALUE_DATA;

typedef int SPM_ChangeCallBackFunction (
    SPMCHANGE_CONTEXT context,      // context handle used to insert values to modified when password is set
    unicode *objectDN,              // DN of the user
    unicode *passwordPolicyDN,      // DN of password policy effective for the user
    nuint32 pwdPolicyConfigOptions, // flags from nspmConfigurationOptions the password policy (see flag definitions above)
    nuint32 flags,                  // Password set flags (see flag definitions above)
    size_t candidatePasswordLen,    // length of password in characters including the NULL termination
    unicode *candidatePassword);    // password that will be set as the new password


#define SPM_CHANGE_FUNCTION_VERSION 0x1

typedef struct SPM_ChangeFunctions
{
    int structureVersion;
    int (*registerAgent)(unicode *agentName);
    int (*deregisterAgent)(unicode *agentName);
    int (*registerChangeCallback)(unicode *agentName, SPM_ChangeCallBackFunction *changeCallback);
    int (*insertModValue)(SPMCHANGE_CONTEXT context, int operation, SPMDS_ATTRIBUTE *attr, void *value, size_t valueSize);
    int (*readAttributeValues)(SPMCHANGE_CONTEXT context, unicode *dn, SPMDS_ATTRIBUTE *attrs, nuint32 numAttrs, SPMDS_VALUES *ds_values, int *numValuesRead);
    int (*freeAttributeValues)(SPMCHANGE_CONTEXT context, SPMDS_VALUES ds_values);
    int (*getValueData)(SPMCHANGE_CONTEXT context, SPMDS_VALUES ds_values, SPMDS_VALUE_DATA *ds_value_data);
    int (*freeValueData)(SPMCHANGE_CONTEXT context, SPMDS_VALUE_DATA *ds_value_data);
} SPM_ChangeFunctions;

NMASAPI int SPM_ChangeAgent(
    unicode *agentName,
    int version,
    SPM_ChangeFunctions *functions);

/* ********************************************************************************************** */
//
// SPM Miscellaneous API
//
/* ********************************************************************************************** */

NMASAPI int SPM_AdministratorSetCredCount(
    unicode *treeName,
    unicode *objectDN,
    nuint32 *adminChangeCount,
    nuint32 *whenAdminLastSet);

NMASAPI int SPM_GetPasswordPolicyDN(
    unicode *treeName,
    unicode *objectDN,
    size_t *passwordPolicyDNLen,  // number of bytes
    unicode *passwordPolicyDN,
    nuint32 *configOptions);

NMASAPI int SPM_GetPasswordPolicyDNWithContext(
    int context,
    size_t *passwordPolicyDNLen,  // number of unicode characters
    unicode *passwordPolicyDN);

NMASAPI int SPM_GetPasswordStatus(
    unicode *subjDN,
    unicode *treeName,
    unicode *objectDN,
    nuint32 *universalPasswordStatus,
    nuint32 *simplePasswordStatus);

NMASAPI int SPM_GenerateRandomUserPassword(
    unicode *treeName,
    unicode *objectDN,
    size_t *passwordLen,  // number of unicode characters including the null
    unicode *password);

NMASAPI int SPM_GenerateRandomPolicyPasswordInit(
    unicode *policyDN,         //DN of password policy object
    void **pHandle);           //receives handle for use with other functions

//the following should return the max password chars needed if the buffer
//parameter is 0, otherwise will generate a password
NMASAPI int SPM_GenerateRandomPolicyPassword(
    void *pHandle,            //handle from init function
    size_t *passwordLen,      //in: length in unicode chars of pasword buffer, out: password char count
    unicode *password);       //buffer for password (may be NULL)

NMASAPI int SPM_GenerateRandomPolicyPasswordCleanup(
     void *pHandle);            //handle from init function

#endif

/************************************************************************/
/************************************************************************/
